package game;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

/**
 * Client.java
 * Class that discovers all bluetooth devices in the neighbourhood
 * and displays their name and bluetooth address.
 * @author Drag-on
 */
public class Client implements DiscoveryListener, Runnable{
    
    //object used for waiting
    private static Object lock=new Object();                // lock
    //vector containing the devices discovered
    private static Vector vecDevices=new Vector();          // vector of found devices
    private StreamConnection connection;                    // connection stream
    private DiscoveryAgent agent=null;                      // agent
    private boolean ServerFinded=false;                     // true if server was found
    public Writer writer;                                   // objet to write to client
    public Vector vectReader;                               // objects vector to read received data
    private Thread t;                                       // current thread
    
    /** Creates a new instance of Client */
    public Client(){
        t=new Thread(this);
        t.start();
        writer=new Writer();
        vectReader=new Vector();
        Reader r=new Reader();
        vectReader.addElement(r);
    }
    
    /** main loop for the thread */
    public void run() {
        //display local device address and name
        try{
            LocalDevice localDevice = LocalDevice.getLocalDevice();
            System.out.println("Address: "+localDevice.getBluetoothAddress());
            System.out.println("Name: "+localDevice.getFriendlyName());
            //find devices
            agent = localDevice.getDiscoveryAgent();
            System.out.println("Starting device inquiry...");
            agent.startInquiry(DiscoveryAgent.GIAC, this);
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            synchronized(lock){
                lock.wait();
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        findService();
    }
    
    /** search the Juno service proposed by the server */
    public void findService(){
        int deviceCount=vecDevices.size();
        if(deviceCount <= 0){
            System.out.println("No Devices Founded .");
        } else{
            // print bluetooth device addresses and names in the format [ No. address (name) ]
            System.out.println(vecDevices.size()+" Bluetooth Devices");
            System.out.println("recherche du serveur");
            for (int i = 0; i <deviceCount; i++) {
                try {
                    agent.searchServices(null, new UUID[] { Server.SERVICE_UUID }, (RemoteDevice)vecDevices.elementAt(i), this );
                }catch (BluetoothStateException e) {
                    e.printStackTrace();
                    if (!ServerFinded)System.out.println("serveur non trouv");} catch (NullPointerException e){}
            }
        }
    }
    
    
    /** This call back method will be called for each discovered bluetooth devices. */
    public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
        System.out.println("Device discovered: "+btDevice.getBluetoothAddress());
        //add the device to the vector
        if(!vecDevices.contains(btDevice)){
            vecDevices.addElement(btDevice);
        }
    }
    
    /** find the desired service and create the connection with server */
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
        if (!ServerFinded){
            ServerFinded=true;
            String url = servRecord[0].getConnectionURL( ServiceRecord.NOAUTHENTICATE_NOENCRYPT, true ); // must be master
            System.out.println("connection :"+url);
            try{
                connection = (StreamConnection) Connector.open( url );
                System.out.println(connection.toString());
                Reader reader=(Reader)vectReader.lastElement();
                reader.in=connection.openDataInputStream();
                DataOutputStream out=connection.openDataOutputStream();
                writer.outPut.addElement(out);
                ConnectionWaitCanvas.num_connected++;
            }catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
    
    /** end of service search */
    public void serviceSearchCompleted(int transID, int respCode) {
    }
    
    /** This callback method will be called when the device discovery is completed. */
    public void inquiryCompleted(int discType) {
        synchronized(lock){
            lock.notify();
        }
        switch (discType) {
            case DiscoveryListener.INQUIRY_COMPLETED :
                System.out.println("INQUIRY_COMPLETED");
                break;
            case DiscoveryListener.INQUIRY_TERMINATED :
                System.out.println("INQUIRY_TERMINATED");
                break;
            case DiscoveryListener.INQUIRY_ERROR :
                System.out.println("INQUIRY_ERROR");
                break;
            default :
                System.out.println("Unknown Response Code");
                break;
        }
    }    
    
    /** disconnect the client */
    public void disconnect(){
        try {
            byte[] b=new byte[1];
            writer.write(b);
            writer.disconnect();
            Reader reader=(Reader)vectReader.lastElement();
            reader.disconnect();
            connection.close();
        } catch (IOException ex) {
            System.out.println("Erreur de fermeture du client");
        }
    }
    
}