package game;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.Sprite;

/**
 * ConnectionWaitCanvas.java
 * Wait screen for client/server connection
 * @author Noda
 */
public final class ConnectionWaitCanvas extends GameCanvas implements Runnable {
    
    // canvas attributes
    private final int timeStep = 40;                 // duration of a frame (in milliseconds)
    private volatile boolean mRun;
    private int curr_frame;
    
    // wait screen attributes
    private Sprite main_bg;
    private Sprite ok_button;
    private CustomFont fnt = CustomFont.getFont("/images/lucidaFontTransp.png", Font.STYLE_PLAIN);
    
    // connection attributes
    public static int num_connected;
    public Server s=null;
    public Client c=null;
    
    /** Creates a new instance of ConnectionWaitCanvas */
    public ConnectionWaitCanvas() {
        super(false);
        try {
            main_bg = new Sprite((Image) Image.createImage("/images/waitScreen.png"), 240, 320);
            ok_button = new Sprite((Image) Image.createImage("/images/mainPressOK.png"), 70, 38);
            ok_button.setPosition(152, 282);
            int[] ok_seq = { 0, 1, 2, 3, 4, 3, 2, 1 };
            ok_button.setFrameSequence(ok_seq);
            num_connected = 0;
            if (JunoMidlet.isServer)createServer();
            else joinServer();
            for(int i=0; i<8; i++)
                SoundManager.freePlayer(i);
            System.gc();
            SoundManager.createPlayer(0, "/sounds/menu/wait.mp3");
            SoundManager.play(0);
            SoundManager.setVolume(0, 70);
            SoundManager.setLoop(0, true);
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /** initialize the game thread */
    public final void start() {
        mRun = true;
        curr_frame = 0;
        Thread t = new Thread(this);
        t.start();
    }
    
    /** main loop for the game thread */
    public final void run() {
        int n = 0;
        Graphics g = getGraphics();
        
        while (mRun) {
            long start = System.currentTimeMillis();
            input();
            render(g);
            curr_frame++;
            long end = System.currentTimeMillis();
            int duration = (int)(end - start);
            if (duration < timeStep) {
                try {
                    Thread.sleep(timeStep - duration);
                } catch (InterruptedException ie) {
                    stop();
                }
            }
        }
        
        // free memory
        main_bg = null;
        ok_button = null;
        for(int i=0; i<8; i++)
            SoundManager.freePlayer(i);
        System.gc();
        JunoGameCanvas.nbClients=nbClients();
        if (JunoMidlet.isServer){
            JunoGameCanvas.r=s.vectReader;
            JunoGameCanvas.w=s.writer;
        } else{JunoGameCanvas.r=c.vectReader;
        JunoGameCanvas.w=c.writer;
        }
        JunoGameCanvas gc = JunoMidlet.midlet.get_Game();
        JunoMidlet.midlet.getDisplay().setCurrent(gc);
        gc.start();
    }
    
    /** key handler for game buttons */
    private final void input() {
        int keyStates = getKeyStates();
        if ((keyStates & FIRE_PRESSED) != 0) {
            if (JunoMidlet.isServer && num_connected > 0) {
                startGame();
                stop();
            }
        }
    }
    
    /** render graphics to screen */
    private final void render(Graphics g) {
        try {
            main_bg.paint(g);
            fnt.setStyle(Font.STYLE_BOLD);
            if (JunoMidlet.isServer) {
                fnt.drawString(g, "Waiting for clients", 120, 100, g.TOP | g.HCENTER);
                
                fnt.setStyle(Font.STYLE_PLAIN);
                fnt.drawString(g, num_connected+" player(s)", 120, 134, g.TOP | g.HCENTER);
                fnt.drawString(g, "are ready", 120, 134+fnt.getHeight()+3, g.TOP | g.HCENTER);
            } else {
                if (num_connected == 0) {
                    fnt.drawString(g, "Connecting...", 120, 100, g.TOP | g.HCENTER);
                } else {
                    fnt.drawString(g, "Connected !", 120, 100, g.TOP | g.HCENTER);
                    fnt.setStyle(Font.STYLE_PLAIN);
                    fnt.drawString(g, "Waiting for", 120, 134, g.TOP | g.HCENTER);
                    fnt.drawString(g, "other players", 120, 134+fnt.getHeight()+3, g.TOP | g.HCENTER);
                    if (!JunoMidlet.isServer && num_connected==1)stop();
                }
            }
            
            if ((curr_frame % 30) < 10) {
                fnt.drawString(g, "*", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            } else if ((curr_frame % 30) < 20) {
                fnt.drawString(g, "**", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            } else {
                fnt.drawString(g, "***", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            }
            // press OK flashing
            if (JunoMidlet.isServer && num_connected > 0) {
                if ((curr_frame % 4) == 0)
                    ok_button.nextFrame();
                ok_button.paint(g);
            }
            System.gc();
            flushGraphics();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /** stop the running thread */
    public final void stop() {
        mRun = false;
    }
    
    /** creates the server */
    public void createServer(){
        s=new Server();
    }
    
    /** join a server */
    public void joinServer(){
        c=new Client();
    }
    
    /** return the number of clients */
    public int nbClients() {
        if (s!=null)
            return s.connection.size();
        else return 0;
    }
    
    /** start game */
    public void startGame(){
        if (JunoMidlet.isServer){s.startgame();}
    }
    
}


