package game;

import java.io.InputStream;
import java.util.Vector;
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.Sprite;
import javax.microedition.media.Manager;
import javax.microedition.media.control.VideoControl;

/**
 * JunoGameCanvas.java
 * Game canvas for the game engine.
 * @author Noda
 */
public final class JunoGameCanvas extends GameCanvas implements Runnable {
    
    // constants
    public final int GAME_WIDTH = getHeight();       // width of the scene
    public final int GAME_HEIGHT = getWidth();       // height of the scene
    private final int timeStep = 40;                 // duration of a frame (in milliseconds)
    private final int RADAR_WIDTH = 238;             // width of the radar
    private final int RADAR_HEIGHT = 274;            // height of the radar
    private final int VIEW_WIDTH = 238;              // width of the view
    private final int VIEW_HEIGHT = 209;             // height of the view
    private final int RADAR_SCROLL_SPEED = 8;        // radar scroll speed
    private final int PHASE_TIME = 30;               // max duration for a phase (in seconds)
    private final int SOFTKEY_LEFT = -6;             // code for left soft key
    private final int SOFTKEY_RIGHT = -7;            // code for right soft key
    private final int SND_OK = 0;                    // sound player id for OK sound
    private final int SND_MUSIC = 7;                 // sound player id for music
    
    // **** scene objects ****
    // Common
    private volatile boolean mRun;
    private Mission mission;
    private char curr_frame;
    private Image shipImage;
    private Image shipImageSmall;
    private Image shipResizedImage;
    private Image radarImage;
    private Sprite ship;
    private Sprite faceSet;
    private Sprite faceIconSet;
    private Image faceIconSetImage;
    private Image faceIconSetImageSmall;
    private Sprite radar_bg;
    private Image selectorImage;
    private Image selectorImageMini;
    private Image selectorCommImage;
    private Image selectorCommImageMini;
    private Sprite selector;
    private float timer = -1f;
    private CustomFont fnt = CustomFont.getFont("/images/lucidaFontTransp.png", Font.STYLE_PLAIN);
    private CustomFont fntMini = CustomFont.getFont("/images/lucidaFontTranspMini.png", Font.STYLE_PLAIN);
    private boolean newPress;
    private boolean wait, wait2, scrollGo;
    private int scrollStart;
    private boolean radarActive;
    private int radarCanActivate;
    
    // INIT_PRESENTATION
    private Sprite INIT_PRESENTATION_bg;
    private Sprite INIT_PRESENTATION_pressKey;
    
    // INIT_RADAR
    private Sprite INIT_RADAR_pressAction;
    private Sprite radarScan;
    private float radar_zoom = 1.0f;
    private int radar_x = 0, radar_y = 0;
    
    // MOVE & ACTION
    private Sprite main_bg;
    private Image buttonsImage;
    private Image buttonsImageRed;
    private Sprite mainPressOK;
    private Sprite mainPressRadar;
    private int moveTo;
    private int actionToDo;
    private Image actionIconsImage;
    
    // REACTION
    private Sprite soundWave;
    private int react_id;
    private int wstart_frame;
    private int wave_x, wave_y;
    
    // VOTE
    private int curr_vote;
    private Sprite arrow;
    
    // END
    private Sprite end_bg;
    private Sprite death;
    
    // connection
    public static boolean isServer=true;
    public static int nbClients=0;
    public static Vector r=null;
    public static Writer w=null;
    
    /** creates a new instance of JunoGameCanvas */
    public JunoGameCanvas() {
        super(false);
        // create the mission
        mission = new Mission(JunoMidlet.isServer, MissionType.CLASSIC,nbClients,r,w);
        // init game
        radarActive = false;
        newPress = true;
        SoundManager.setLoop(0, false);
        SoundManager.createPlayer(SND_OK, "/sounds/ok.amr");
        SoundManager.setVolume(SND_OK, 80);
        radarCanActivate = mission.type.radar_delay + 1;
        initPhase();
    }
    
    /** initialize the game thread */
    public final void start() {
        mRun = true;
        curr_frame = 0;
        Thread t = new Thread(this);
        t.start();
    }
    
    /** main loop for the game thread */
    public final void run() {
        int n = 0;
        Graphics g = getGraphics();
        try {
            while (mRun) {
                long start = System.currentTimeMillis();
                tick();
                input();
                render(g);
                curr_frame++;
                long end = System.currentTimeMillis();
                int duration = (int)(end - start);
                if (duration < timeStep) {
                    try {
                        Thread.sleep(timeStep - duration);
                    } catch (InterruptedException ie) {
                        stop();
                    }
                }
            }
        } catch (Exception e) {
            JunoMidlet.midlet.alertError(e.toString());
        }
        // play ending video
        javax.microedition.media.Player videoPlayer = null;
        VideoControl video = null;
        try {
            // get the video
            InputStream is = null;
            
            if (!mission.players[0].alive)
                is = JunoMidlet.class.getResourceAsStream("/videos/death.3gp");
            else {
                switch (MissionType.checkWin(mission.getAliens(), mission.getTechnicians(), mission.getWreckers(), mission.getDestroyed(), mission.turn)) {
                    case MissionType.GOODTEAM_WINS: is = JunoMidlet.class.getResourceAsStream("/videos/end_mission_complete.3gp"); break;
                    case MissionType.ALIENS_WINS: is = JunoMidlet.class.getResourceAsStream("/videos/end_alien_win.3gp"); break;
                    case MissionType.WRECKERS_WINS: is = JunoMidlet.class.getResourceAsStream("/videos/end_wreckers_win.3gp"); break;
                }
            }
            
            if (is != null) {
                videoPlayer = Manager.createPlayer(is, "video/3gpp");
                videoPlayer.realize();
                videoPlayer.prefetch();
                // get the video controller
                video = (VideoControl) videoPlayer.getControl("VideoControl");
                if (video == null)
                    throw new Exception("Video control not supported!");
                // put the video on the current display canvas
                video.initDisplayMode(VideoControl.USE_DIRECT_VIDEO, this);
                video.setVisible(true);
//                video.setDisplaySize(176, 220);
//                video.setDisplayLocation(0, 0);
                // start the video
                videoPlayer.start();
            }
        } catch(Exception e) {
            System.out.println(e);
        }
        // wait until video ends
        while(videoPlayer.getState() == javax.microedition.media.Player.STARTED);
        // stop video player
        try {
            if (videoPlayer != null && video != null) {
                try {
                    videoPlayer.stop();
                } catch (Exception e) {
                    
                }
                videoPlayer.deallocate();
                videoPlayer.close();
            }
        } catch (Exception e) {}
        System.gc();
        
        // return to main menu
        JunoMidlet.midlet.getDisplay().setCurrent(JunoMidlet.midlet.get_mainMenu().getSvgCanvas());
        SoundManager.createPlayer(0, "/sounds/menu/menu.mp3");
        SoundManager.setLoop(0, true);
        SoundManager.play(0);
    }
    
    /** actions to do each frame */
    private final void tick() {
        if (timer > 0 && !wait) {
            timer -= ((float)timeStep)/1000.0;
            if (timer < 0)
                timer = 0;
        }
        
        if (timer == 0) {
            switch(mission.phase) {
                case MissionPhase.MOVE:
                    mission.nextPhase();
                    initPhase();
                    break;
                case MissionPhase.ACTION:
                    mission.nextPhase();
                    initPhase();
                    break;
                case MissionPhase.REACT:
                    mission.nextPhase();
                    initPhase();
                    break;
                case MissionPhase.STATS:
                    mission.nextPhase();
                    initPhase();
                    break;
                case MissionPhase.VOTE:
                    mission.players[0].vote = mission.getPlayerID(curr_vote, 0);
                    mission.nextPhase();
                    initPhase();
                    break;
                case MissionPhase.COMMANDER_ACTION:
                    if (mission.players[0].type == PlayerType.COMMANDER) {
                        mission.eliminate = mission.getPlayerID(curr_vote, 0);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.DECISION:
                    mission.nextPhase();
                    initPhase();
                    break;
            }
        }
        
        mission.tick();
        if (mission.ready) {
            switch(mission.phase) {
                case MissionPhase.WAIT_PLAYERS0: mission.nextPhase(); initPhase(); break;
                case MissionPhase.WAIT_PLAYERS1: mission.nextPhase(); initPhase(); break;
                case MissionPhase.WAIT_PLAYERS2: mission.nextPhase(); initPhase(); break;
                case MissionPhase.COMMANDER_ACTION:
                    if (mission.players[0].type != PlayerType.COMMANDER && curr_frame > 40) {
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
            }
        }
    }
    
    /** key handler for fast game buttons */
    private final void input() {
        int keyStates = getKeyStates();
        if ((keyStates & LEFT_PRESSED) != 0) {
            switch(mission.phase) {
                case MissionPhase.INIT_RADAR:
                case MissionPhase.MOVE:
                case MissionPhase.ACTION:
                case MissionPhase.REACT:
                    radar_x -= RADAR_SCROLL_SPEED;
                    if (radar_x < 0)
                        radar_x = 0;
                    break;
            }
        } else if ((keyStates & RIGHT_PRESSED) != 0) {
            switch(mission.phase) {
                case MissionPhase.INIT_RADAR:
                    radar_x += RADAR_SCROLL_SPEED;
                    int lim = (int)(shipImage.getWidth()-RADAR_WIDTH);
                    if (radar_x >= lim)
                        radar_x = lim-1;
                    break;
                case MissionPhase.MOVE:
                case MissionPhase.ACTION:
                case MissionPhase.REACT:
                    radar_x += RADAR_SCROLL_SPEED;
                    int lim2 = (int)(shipImage.getWidth()-VIEW_WIDTH);
                    if (radar_x >= lim2)
                        radar_x = lim2-1;
                    break;
            }
        } else if ((keyStates & UP_PRESSED) != 0) {
            switch(mission.phase) {
                case MissionPhase.INIT_RADAR:
                case MissionPhase.MOVE:
                case MissionPhase.ACTION:
                case MissionPhase.REACT:
                    radar_y -= RADAR_SCROLL_SPEED;
                    if (radar_y < 0)
                        radar_y =0;
                    break;
                case MissionPhase.VOTE:
                case MissionPhase.COMMANDER_ACTION:
                    if (newPress)
                        curr_vote--;
                    if (curr_vote < 0)
                        curr_vote = mission.getMaxVote(0)-1;
                    break;
            }
        } else if ((keyStates & DOWN_PRESSED) != 0) {
            switch(mission.phase) {
                case MissionPhase.INIT_RADAR:
                    radar_y += RADAR_SCROLL_SPEED;
                    int lim = (int)(shipImage.getHeight()-RADAR_HEIGHT);
                    if (radar_y >= lim)
                        radar_y = lim-1;
                    break;
                case MissionPhase.MOVE:
                case MissionPhase.ACTION:
                case MissionPhase.REACT:
                    radar_y += RADAR_SCROLL_SPEED;
                    int lim2 = (int)(shipImage.getHeight()-VIEW_HEIGHT);
                    if (radar_y >= lim2)
                        radar_y = lim2-1;
                    break;
                case MissionPhase.VOTE:
                case MissionPhase.COMMANDER_ACTION:
                    if (newPress)
                        curr_vote = (curr_vote+1) % mission.getMaxVote(0);
                    break;
            }
        } else if ((keyStates & FIRE_PRESSED) != 0) {
            switch(mission.phase) {
                case MissionPhase.INIT_RADAR:
                    if (newPress && !wait) {
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.MOVE:
                    if (moveTo != -1) {
                        SoundManager.play(SND_OK);
                        mission.players[0].inRoom = Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + moveTo];
                        mission.nextPhase();
                        initPhase();
                    } else if (!wait && newPress) {
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.ACTION:
                    if (!wait && newPress) {
                        if (actionToDo != MissionType.ACTION_NOACTION) {
                            if (actionToDo == MissionType.ACTION_DESTROY)
                                mission.destroyed[mission.players[0].inRoom] = true;
                            else if (actionToDo == MissionType.ACTION_REPAIR)
                                mission.destroyed[mission.players[0].inRoom] = false;
                            mission.activity[mission.players[0].inRoom] = true;
                            mission.players[0].actions_left[actionToDo]--;
                        }
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.REACT:
                    if (!wait2 && newPress) {
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.STATS:
                    if (!wait && newPress) {
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.VOTE:
                    if (newPress) {
                        mission.players[0].vote = mission.getPlayerID(curr_vote, 0);
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.COMMANDER_ACTION:
                    if (mission.players[0].type == PlayerType.COMMANDER && newPress) {
                        mission.eliminate = mission.getPlayerID(curr_vote, 0);
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.DECISION:
                    if (newPress) {
                        SoundManager.play(SND_OK);
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
                case MissionPhase.END:
                    if (newPress && !wait) {
                        stop();
                    }
                    break;
            }
        } else if ((keyStates & GAME_A_PRESSED) != 0) {
            switch (mission.phase) {
                case MissionPhase.INIT_RADAR:
                    if (radar_zoom == 1.0) {
                        radar_zoom = 0.5f;
                        shipResizedImage = shipImageSmall;
                    }
                    break;
                case MissionPhase.MOVE:
                    moveTo = 0;
                    break;
            }
            
        } else if ((keyStates & GAME_B_PRESSED) != 0) {
            switch (mission.phase) {
                case MissionPhase.INIT_RADAR:
                    if (radar_zoom == 0.5) {
                        radar_zoom = 1.0f;
                        shipResizedImage = shipImage;
                    }
                    break;
                case MissionPhase.MOVE:
                    moveTo = 2;
                    break;
            }
        }
        if (keyStates != 0) {
            switch (mission.phase) {
                case MissionPhase.INIT_PRESENTATION:
                    if (!wait) {
                        mission.nextPhase();
                        initPhase();
                    }
                    break;
            }
            newPress = false;
        } else {
            newPress = true;
        }
    }
    
    /** key handler for special keys */
    public final void keyPressed(int keycode) {
        if (keycode != 0) {
            if(keycode < 0) {
                // check soft keys
                if(keycode == SOFTKEY_LEFT) {
                    switch(mission.phase) {
                        case MissionPhase.MOVE:
                            if(radarCanActivate == 0) {
                                SoundManager.play(2);
                                radarActive = true;
                                radarCanActivate = mission.type.radar_delay;
                                curr_frame = 0;
                            }
                            break;
                    }
                } else if(keycode == SOFTKEY_RIGHT) {
                    switch(mission.phase) {
                        case MissionPhase.MOVE:
                            if(moveTo != -1) {
                                mission.players[0].inRoom = Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + moveTo];
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            } else if (!wait) {
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.ACTION:
                            if (!wait) {
                                if (actionToDo != MissionType.ACTION_NOACTION) {
                                    if (actionToDo == MissionType.ACTION_DESTROY)
                                        mission.destroyed[mission.players[0].inRoom] = true;
                                    else if (actionToDo == MissionType.ACTION_REPAIR)
                                        mission.destroyed[mission.players[0].inRoom] = false;
                                    mission.activity[mission.players[0].inRoom] = true;
                                    mission.players[0].actions_left[actionToDo]--;
                                }
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.REACT:
                            if (!wait2) {
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.STATS:
                            if (!wait) {
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.VOTE:
                            if (newPress) {
                                mission.players[0].vote = mission.getPlayerID(curr_vote, 0);
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.COMMANDER_ACTION:
                            if (mission.players[0].type == PlayerType.COMMANDER) {
                                mission.eliminate = mission.getPlayerID(curr_vote, 0);
                                SoundManager.play(SND_OK);
                                mission.nextPhase();
                                initPhase();
                            }
                            break;
                        case MissionPhase.DECISION:
                            SoundManager.play(SND_OK);
                            mission.nextPhase();
                            initPhase();
                            break;
                        case MissionPhase.END:
                            if (!wait) {
                                stop();
                            }
                            break;
                    }
                }
            } else {
                switch(keycode) {
                    case KEY_NUM1:
                        switch(mission.phase) {
                            case MissionPhase.INIT_RADAR:
                                if(radar_zoom == 1.0) {
                                    radar_zoom = 0.5f;
                                    shipResizedImage = shipImageSmall;
                                }
                                break;
                            case MissionPhase.MOVE:
                                moveTo = 0;
                                break;
                            case MissionPhase.ACTION:
                                if (actionToDo == MissionType.ACTION_NOACTION)
                                    actionToDo = MissionType.getActionCode(mission.players[0], mission.destroyed[mission.players[0].inRoom], 1);
                                else
                                    actionToDo = MissionType.ACTION_NOACTION;
                                scrollGo = false;
                                scrollStart = curr_frame;
                                break;
                        }
                        break;
                    case KEY_NUM2:
                        switch(mission.phase) {
                            case MissionPhase.MOVE:
                                moveTo = 1;
                                break;
                        }
                        break;
                    case KEY_NUM3:
                        switch(mission.phase) {
                            case MissionPhase.INIT_RADAR:
                                if(radar_zoom == 0.5) {
                                    radar_zoom = 1.0f;
                                    shipResizedImage = shipImage;
                                }
                                break;
                            case MissionPhase.MOVE:
                                moveTo = 2;
                                break;
                                
                        }
                        break;
                }
            }
        }
    }
    
    /** render graphics to screen */
    private final void render(Graphics g) {
        // the code below can be used to resize the game screen to any size,
        // just uncomment it and rename the parameter 'g' above into 'gfx'
/*        final boolean RESIZE = true;

        Graphics g;
        Image test;
        if(RESIZE) {
            test = Image.createImage(240, 320);
            g = test.getGraphics();
        } else {
            g = gfx;
        }
*/        
        try {
            switch(mission.phase) {
                case MissionPhase.INIT_PRESENTATION: INIT_PRESENTATION_paint(g); break;
                case MissionPhase.INIT_RADAR: INIT_RADAR_paint(g); break;
                case MissionPhase.MOVE: MOVE_paint(g); break;
                case MissionPhase.WAIT_PLAYERS0: WAIT_PLAYERS012_paint(g); break;
                case MissionPhase.ACTION: ACTION_paint(g); break;
                case MissionPhase.WAIT_PLAYERS1: WAIT_PLAYERS012_paint(g); break;
                case MissionPhase.REACT: REACT_paint(g); break;
                case MissionPhase.STATS: STATS_paint(g); break;
                case MissionPhase.VOTE: VOTE_paint(g); break;
                case MissionPhase.WAIT_PLAYERS2: WAIT_PLAYERS012_paint(g); break;
                case MissionPhase.COMMANDER_ACTION: COMMANDER_ACTION_paint(g); break;
                case MissionPhase.DECISION: DECISION_paint(g); break;
                case MissionPhase.END: END_paint(g); break;
            }
/*            
            if(RESIZE)
                gfx.drawImage(resizeImage(test, 176, 220), 0, 0, Graphics.LEFT | Graphics.TOP);
*/            
            flushGraphics();
            
            if (curr_frame % 25 == 0)
                System.gc();
            
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /** stop the running thread */
    public final void stop() {
        mRun = false;
    }
    
    /** resize an image */
    private static final Image resizeImage(Image src, int screenWidth, int screenHeight) {
        int srcWidth = src.getWidth();
        int srcHeight = src.getHeight();
        Image tmp = Image.createImage(screenWidth, srcHeight);
        Graphics g = tmp.getGraphics();
        int ratio = (srcWidth << 16) / screenWidth;
        int pos = ratio/2;
        
        for (int x = 0; x < screenWidth; x++) {
            g.setClip(x, 0, 1, srcHeight);
            g.drawImage(src, x - (pos >> 16), 0, Graphics.LEFT | Graphics.TOP);
            pos += ratio;
        }
        
        Image resizedImage = Image.createImage(screenWidth, screenHeight);
        g = resizedImage.getGraphics();
        ratio = (srcHeight << 16) / screenHeight;
        pos = ratio/2;
        
        for (int y = 0; y < screenHeight; y++) {
            g.setClip(0, y, screenWidth, 1);
            g.drawImage(tmp, 0, y - (pos >> 16), Graphics.LEFT | Graphics.TOP);
            pos += ratio;
        }
        
        return resizedImage;
    }
    
    /** clip an image */
    private static final Image clip(Image img, int clipX, int clipY, int width, int height) {
        return Image.createImage(img, clipX, clipY, width, height, Sprite.TRANS_NONE);
    }
    
    /** print a scrolling text (if necessary) */
    private final void drawScrollingString(Graphics g, String str, int x, int y) {
        if(!wait && (curr_frame == scrollStart+40))
            scrollGo = true;
        
        if (str.length() > 25) {
            if (scrollGo) {
                int str_start = ((curr_frame-scrollStart-40)/4) % str.length();
                int str_end = str_start + 25;
                if (str_end > str.length())
                    str_end = str.length();
                if (str_start > (str.length() - 25))
                    fntMini.drawString(g, str.substring(str_start, str_end)+str.substring(0, 25-(str_end-str_start)), x, y, g.TOP | g.LEFT);
                else
                    fntMini.drawString(g, str.substring(str_start, str_end), x, y, g.TOP | g.LEFT);
            } else {
                fntMini.drawString(g, str.substring(0, 25), x, y, g.TOP | g.LEFT);
            }
        } else {
            fntMini.drawString(g, str, x, y, g.TOP | g.LEFT);
        }
    }
    
    /** paint the people located inside given room */
    private final void paintRoom(Graphics g, int room, int x_offset, int y_offset) {
        int num_people = 0;
        int people[] = new int[mission.type.num_players];
        int x = Mission.ROOM_COORD[room << 1] - 16 - x_offset;
        int y = Mission.ROOM_COORD[(room << 1) + 1] - 16 - y_offset;
        
        // find the number of people in this room
        for (int i=0; i<mission.type.num_players; i++)
            if (mission.players[i].alive && mission.players[i].inRoom == room)
                people[num_people++] = i;
        
        // init paint position
        if (num_people > 3) {
            if (room != 9) {
                y -= 17;    // (32 / 2) + (2 / 2)
                x -= 34;    // ((32 / 2) + (2 / 2)) * 2;
            } else {
                switch(num_people) {
                    case 4: x -= 51; break;     // ((32 / 2) + (2 / 2)) + 32 + 2;
                    case 5: x -= 68; break;     // (32 + 2) * 2;
                    case 6: x -= 85; break;     // ((32 / 2) + (2 / 2)) + (32 + 2) * 2;
                }
            }
        } else {
            if (num_people == 2)
                x -= 17;    // (32 / 2) + (2 / 2)
            else if (num_people == 3)
                x -= 34;    // ((32 / 2) + (2 / 2)) * 2;
        }
        
        // paint people
        for(int i=0; i<num_people; i++) {
            if (i == 3) {
                if (room != 9) {
                    y += 34;    // 32 + 2
                    switch(num_people) {
                        case 4: x -= 68; break;     // 2 * (32 + 2)
                        case 5: x -= 85; break;     // (2 * (32 + 2)) + (32 / 2) + (2 / 2)
                        case 6: x -= 102; break;    // 3 * (32 + 2)
                    }
                }
            }
            g.drawRegion(faceIconSetImage, 0, mission.players[people[i]].avatar << 5, 32, 32, Sprite.TRANS_NONE, x, y, Graphics.LEFT | Graphics.TOP);
            x += 34;        // 32 + 2
        }
    }
    
    /** paint people located in rooms around the player */
    private final void paintProximityRadar(Graphics g, int x_offset, int y_offset) {
        for(int i=0; i<3; i++)
            paintRoom(g, Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + i], x_offset, y_offset);
    }
    
    /** return a string to draw bars for votes */
    private static final String toBars(int num) {
        String b = new String("_");
        for (int i=0; i<num; i++)
            b = b + "_";
        return b;
    }
    
    /** initialize game phase */
    private final void initPhase() {
        scrollGo = false;
        curr_frame = 0;
        
        for(int i=1; i<8; i++)
            SoundManager.freePlayer(i);
        System.gc();
        
        Image tmp;
        Graphics g;
        try {
            switch(mission.phase) {
                case MissionPhase.INIT_PRESENTATION:
                    faceSet = new Sprite((Image) Image.createImage("/images/faceSet.png"), 64, 64);
                    INIT_PRESENTATION_bg = new Sprite((Image) Image.createImage("/images/presentationScreen.png"), 240, 320);
                    INIT_PRESENTATION_pressKey = new Sprite((Image) Image.createImage("/images/presentationPressKey.png"), 240, 32);
                    INIT_PRESENTATION_pressKey.setPosition(0, 320-32);
                    wait = true;
                    SoundManager.createPlayer(1, "/sounds/roll.amr");
                    SoundManager.setVolume(1, 70);
                    SoundManager.createPlayer(2, "/sounds/roll_end.amr");
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.INIT_RADAR:
                    // unload images from previous phase
                    faceSet = null;
                    INIT_PRESENTATION_bg = null;
                    INIT_PRESENTATION_pressKey = null;
                    System.gc();
                    
                    radar_bg = new Sprite((Image) Image.createImage("/images/radarScreen.png"), 240, 320);
                    INIT_RADAR_pressAction = new Sprite((Image) Image.createImage("/images/radarPressAction.png"), 90, 42);
                    INIT_RADAR_pressAction.setPosition(0, 278);
                    radarScan = new Sprite((Image) Image.createImage("/images/radarScan.png"), 240, 32);
                    radar_zoom = 1.0f;
                    faceIconSetImage = (Image) Image.createImage("/images/faceIconSet.png");
                    faceIconSetImageSmall = (Image) Image.createImage("/images/faceIconSetMini.png");
                    selectorImage = (Image) Image.createImage("/images/selector.png");
                    selectorImageMini = (Image) Image.createImage("/images/selectorMini.png");
                    selectorCommImage = (Image) Image.createImage("/images/selectorCommander.png");
                    selectorCommImageMini = (Image) Image.createImage("/images/selectorCommanderMini.png");
                    // draw ship image with players in
                    tmp = (Image) Image.createImage("/images/ship.png");
                    shipImage = Image.createImage(tmp.getWidth(), tmp.getHeight());
                    g = shipImage.getGraphics();
                    g.drawImage(tmp, 0, 0, Graphics.LEFT | Graphics.TOP);
                    tmp = null;
                    System.gc();
                    for(int i=0; i<mission.type.num_players; i++) {
                        g.drawRegion(faceIconSetImage, 0, mission.players[i].avatar << 5, 32, 32, Sprite.TRANS_NONE, Mission.ROOM_COORD[mission.players[i].inRoom << 1]-16, Mission.ROOM_COORD[(mission.players[i].inRoom << 1)+1]-16, Graphics.LEFT | Graphics.TOP);
                    }
                    faceIconSetImage = null;
                    // small version
                    tmp = (Image) Image.createImage("/images/shipMini.png");
                    shipImageSmall = Image.createImage(tmp.getWidth(), tmp.getHeight());
                    g = shipImageSmall.getGraphics();
                    g.drawImage(tmp, 0, 0, Graphics.LEFT | Graphics.TOP);
                    tmp = null;
                    System.gc();
                    for(int i=0; i<mission.type.num_players; i++) {
                        g.drawRegion(faceIconSetImageSmall, 0, mission.players[i].avatar << 4, 16, 16, Sprite.TRANS_NONE, (Mission.ROOM_COORD[mission.players[i].inRoom << 1] >> 1)-8, (Mission.ROOM_COORD[(mission.players[i].inRoom << 1)+1] >> 1)-8, Graphics.LEFT | Graphics.TOP);
                    }
                    faceIconSetImageSmall = null;
                    shipResizedImage = shipImage;
                    radar_x = Mission.ROOM_COORD[mission.players[0].inRoom << 1]-(RADAR_WIDTH >> 1);
                    radar_y = Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]-(RADAR_HEIGHT >> 1);
                    if(radar_x < 0) radar_x = 0;
                    if(radar_x > shipResizedImage.getWidth()-RADAR_WIDTH) radar_x = shipResizedImage.getWidth()-RADAR_WIDTH-1;
                    if(radar_y < 0) radar_y = 0;
                    if(radar_y > shipResizedImage.getHeight()-RADAR_HEIGHT) radar_y = shipResizedImage.getHeight()-RADAR_HEIGHT-1;
                    wait = true;
                    SoundManager.createPlayer(1, "/sounds/radar.amr");
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.MOVE:
                    SoundManager.stop(SND_MUSIC);
                    // unload images from previous phase
                    radar_bg = null;
                    shipResizedImage = null;
                    INIT_RADAR_pressAction = null;
                    shipImageSmall = null;
                    selectorCommImage = null;
                    selectorCommImageMini = null;
                    selectorImage = null;
                    selectorImageMini = null;
                    radarScan = null;
                    System.gc();
                    SoundManager.createPlayer(1, "/sounds/phase1_move.amr");
                    SoundManager.createPlayer(2, "/sounds/proximity_radar_activated.amr");
                    SoundManager.createPlayer(SND_MUSIC, Mission.ROOM_AMBIANCE[mission.players[0].inRoom]);
                    SoundManager.setVolume(SND_MUSIC, 80);
                    System.gc();
                    
                    // init radar system
                    if (radarCanActivate > 0)
                        --radarCanActivate;
                    if (radarActive)
                        radarActive = false;
                    
                    radarScan = new Sprite((Image) Image.createImage("/images/radarScan.png"), 240, 32);
                    main_bg = new Sprite((Image) Image.createImage("/images/mainScreen.png"), 240, 320);
                    mainPressOK = new Sprite((Image) Image.createImage("/images/mainPressOK.png"), 70, 38);
                    mainPressOK.setPosition(152, 282);
                    int[] ok_seq = { 0, 1, 2, 3, 4, 3, 2, 1 };
                    mainPressOK.setFrameSequence(ok_seq);
                    mainPressRadar = new Sprite((Image) Image.createImage("/images/mainPressRadar.png"), 82, 24);
                    mainPressRadar.setPosition(8, 296);
                    mainPressRadar.setFrameSequence(ok_seq);
                    faceIconSetImage = (Image) Image.createImage("/images/faceIconSet.png");
                    buttonsImage = (Image) Image.createImage("/images/buttons.png");
                    buttonsImageRed = (Image) Image.createImage("/images/buttonsRed.png");
                    faceIconSet = new Sprite(faceIconSetImage, 32, 32);
                    radar_zoom = 1.0f;
                    // draw ship image with player in
                    tmp = (Image) Image.createImage("/images/ship.png");
                    shipImage = Image.createImage(tmp.getWidth(), tmp.getHeight());
                    g = shipImage.getGraphics();
                    g.drawImage(tmp, 0, 0, Graphics.LEFT | Graphics.TOP);
                    tmp = null;
                    System.gc();
                    paintRoom(g, mission.players[0].inRoom, 0, 0);
                    shipResizedImage = shipImage;
                    radar_x = Mission.ROOM_COORD[mission.players[0].inRoom << 1]-(VIEW_WIDTH >> 1);
                    radar_y = Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]-(VIEW_HEIGHT >> 1);
                    wait = true;
                    wait2 = true;
                    if(radar_x < 0) radar_x = 0;
                    if(radar_x > shipResizedImage.getWidth()-VIEW_WIDTH) radar_x = shipResizedImage.getWidth()-VIEW_WIDTH-1;
                    if(radar_y < 0) radar_y = 0;
                    if(radar_y > shipResizedImage.getHeight()-VIEW_HEIGHT) radar_y = shipResizedImage.getHeight()-VIEW_HEIGHT-1;
                    curr_frame = 0;
                    faceIconSet.setFrame(mission.players[0].avatar);
                    faceIconSet.setPosition(9, 8);
                    timer = PHASE_TIME;
                    moveTo = -1;
                    scrollStart = 0;
                    SoundManager.play(SND_MUSIC);
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.WAIT_PLAYERS0:
                    // unload images from previous phase
                    radarScan = null;
                    mainPressRadar = null;
                    buttonsImage = null;
                    System.gc();
                    break;
                    
                case MissionPhase.ACTION:
                    SoundManager.stop(SND_MUSIC);
                    actionIconsImage = (Image) Image.createImage("/images/actionIcons.png");
                    // draw ship image with player in
                    tmp = (Image) Image.createImage("/images/ship.png");
                    shipImage = Image.createImage(tmp.getWidth(), tmp.getHeight());
                    g = shipImage.getGraphics();
                    g.drawImage(tmp, 0, 0, Graphics.LEFT | Graphics.TOP);
                    tmp = null;
                    System.gc();
                    SoundManager.createPlayer(1, "/sounds/phase2_action.amr");
                    SoundManager.createPlayer(SND_MUSIC, Mission.ROOM_AMBIANCE[mission.players[0].inRoom]);
                    SoundManager.setVolume(SND_MUSIC, 80);
                    System.gc();
                    
                    paintRoom(g, mission.players[0].inRoom, 0, 0);
                    shipResizedImage = shipImage;
                    wait = true;
                    radar_x = Mission.ROOM_COORD[mission.players[0].inRoom << 1]-(VIEW_WIDTH >> 1);
                    radar_y = Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]-(VIEW_HEIGHT >> 1);
                    if(radar_x < 0) radar_x = 0;
                    if(radar_x > shipResizedImage.getWidth()-VIEW_WIDTH) radar_x = shipResizedImage.getWidth()-VIEW_WIDTH-1;
                    if(radar_y < 0) radar_y = 0;
                    if(radar_y > shipResizedImage.getHeight()-VIEW_HEIGHT) radar_y = shipResizedImage.getHeight()-VIEW_HEIGHT-1;
                    curr_frame = 0;
                    timer = PHASE_TIME;
                    actionToDo = MissionType.ACTION_NOACTION;
                    for(int i=0; i<Mission.NUM_ROOMS; i++)
                        mission.activity[i] = false;
                    scrollStart = 0;
                    SoundManager.play(SND_MUSIC);
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.WAIT_PLAYERS1:
                    actionIconsImage = null;
                    System.gc();
                    break;
                    
                case MissionPhase.REACT:
                    SoundManager.stop(SND_MUSIC);
                    SoundManager.createPlayer(1, "/sounds/phase3_reaction.amr");
                    SoundManager.createPlayer(SND_MUSIC, Mission.ROOM_AMBIANCE[mission.players[0].inRoom]);
                    SoundManager.setVolume(SND_MUSIC, 50);
                    SoundManager.createPlayer(2, "/sounds/noise/noise1.amr", false);
                    SoundManager.createPlayer(3, "/sounds/noise/noise2.amr", false);
                    SoundManager.createPlayer(4, "/sounds/noise/noise3.amr", false);
                    SoundManager.createPlayer(5, "/sounds/noise/noise4.amr", false);
                    System.gc();
                    
                    curr_frame = 0;
                    wait = true;
                    wait2 = true;
                    scrollStart = 0;
                    react_id = -2;
                    timer = -1;
                    soundWave = new Sprite((Image) Image.createImage("/images/soundWave.png"), 64, 64);
                    soundWave.setVisible(false);
                    wave_x = 0;
                    wave_y = 0;
                    SoundManager.play(SND_MUSIC);
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.STATS:
                    SoundManager.stop(SND_MUSIC);
                    soundWave = null;
                    shipImage = null;
                    shipResizedImage = null;
                    System.gc();
                    
                    timer = PHASE_TIME;
                    curr_frame = 0;
                    wait = true;
                    break;
                    
                case MissionPhase.VOTE:
                    SoundManager.createPlayer(1, "/sounds/phase4_vote.amr");
                    System.gc();
                    
                    arrow = new Sprite((Image) Image.createImage("/images/arrow.png"), 32, 32);
                    int[] arrow_seq = { 0, 1, 2, 3, 4, 5, 4, 3, 2, 1 };
                    arrow.setFrameSequence(arrow_seq);
                    timer = PHASE_TIME;
                    curr_frame = 0;
                    wait = true;
                    curr_vote = 0;
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.WAIT_PLAYERS2:
                    System.gc();
                    break;
                    
                case MissionPhase.COMMANDER_ACTION:
                    timer = PHASE_TIME;
                    curr_frame = 0;
                    curr_vote = 0;
                    System.gc();
                    break;
                    
                case MissionPhase.DECISION:
                    arrow = null;
                    SoundManager.createPlayer(1, "/sounds/phase5_decision.amr");
                    System.gc();
                    
                    timer = PHASE_TIME/2;
                    curr_frame = 0;
                    curr_vote = 0;
                    wait = true;
                    SoundManager.play(1);
                    break;
                    
                case MissionPhase.END:
                    main_bg = null;
                    if (mission.getWin() && mission.players[0].alive)
                        SoundManager.createPlayer(1, "/sounds/you_win.amr");
                    else
                        SoundManager.createPlayer(1, "/sounds/you_lose.amr");
                    if (!mission.players[0].alive)
                        SoundManager.createPlayer(2, "/sounds/dead.amr");
                    System.gc();
                    
                    wait = true;
                    curr_frame = 0;
                    end_bg = new Sprite((Image) Image.createImage("/images/endScreen.png"), 240, 320);
                    death = new Sprite((Image) Image.createImage("/images/death.png"), 176, 203);
                    death.setPosition(36, 69);
                    if (!mission.players[0].alive)
                        SoundManager.play(2);
                    JunoMidlet.midlet.getDisplay().vibrate(1000);
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private final void INIT_PRESENTATION_paint(Graphics g) {
        INIT_PRESENTATION_bg.paint(g);
        faceSet.setPosition(18, 63);
        
        if(curr_frame == 60) {
            wait = false;
            SoundManager.stop(1);
            SoundManager.play(2);
        }
        
        if(wait) {
            faceSet.nextFrame();
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "Checking ID", 162, 80, g.TOP | g.HCENTER);
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, "Please wait...", 120, 176, g.TOP | g.HCENTER);
        } else {
            // text blinking
            if ((curr_frame % 20) <= 10)
                INIT_PRESENTATION_pressKey.paint(g);
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, mission.players[0].name, 162, 64, g.TOP | g.HCENTER);
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 162, 88, g.TOP | g.HCENTER);
            fnt.drawString(g, "Mission :", 20, 148, g.TOP | g.LEFT);
            fnt.setStyle(Font.STYLE_ITALIC);
            fnt.drawString(g, PlayerType.getMission(mission.players[0].type), 20, 166, g.TOP | g.LEFT);
            faceSet.setFrame(mission.players[0].avatar);
        }
        faceSet.paint(g);
    }
    
    private final void INIT_RADAR_paint(Graphics g) {
        radar_bg.paint(g);
        
        if(curr_frame == 40)
            wait = false;
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "Scanning...", 120, 144, g.TOP | g.HCENTER);
        } else {
            // press action key animation
            if ((curr_frame % 4) == 0)
                INIT_RADAR_pressAction.nextFrame();
            // draw space ship
            int x = radar_x*(shipResizedImage.getWidth()-RADAR_WIDTH)/(shipImage.getWidth()-RADAR_WIDTH);
            int y = radar_y*(shipResizedImage.getHeight()-RADAR_HEIGHT)/(shipImage.getHeight()-RADAR_HEIGHT);
            radarImage = clip(shipResizedImage, (int)(x), (int)(y), RADAR_WIDTH, RADAR_HEIGHT);
            // draw current player selector
            if ((curr_frame % 20) < 10) {
                Image tmp = Image.createImage(radarImage.getWidth(), radarImage.getHeight());
                Graphics gt = tmp.getGraphics();
                gt.drawImage(radarImage, 0, 0, Graphics.LEFT | Graphics.TOP);
                gt.setClip(0, 0, RADAR_WIDTH, RADAR_HEIGHT);
                if (radar_zoom == 1.0) {
                    int player_x = (int)(Mission.ROOM_COORD[mission.players[0].inRoom << 1]*radar_zoom)-(selectorImage.getWidth() >> 1)-x;
                    int player_y = (int)(Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]*radar_zoom)-(selectorImage.getHeight() >> 1)-y;
                    gt.drawImage(selectorImage, player_x, player_y, Graphics.LEFT | Graphics.TOP);
                    // commander
                    player_x = (int)(Mission.ROOM_COORD[mission.players[mission.commander].inRoom << 1]*radar_zoom)-(selectorCommImage.getWidth() >> 1)-x;
                    player_y = (int)(Mission.ROOM_COORD[(mission.players[mission.commander].inRoom << 1)+1]*radar_zoom)-(selectorCommImage.getHeight() >> 1)-y-6;
                    gt.drawImage(selectorCommImage, player_x, player_y, Graphics.LEFT | Graphics.TOP);
                } else {
                    int player_x = (int)(Mission.ROOM_COORD[mission.players[0].inRoom << 1]*radar_zoom)-(selectorImageMini.getWidth() >> 1)-x;
                    int player_y = (int)(Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]*radar_zoom)-(selectorImageMini.getHeight() >> 1)-y;
                    gt.drawImage(selectorImageMini, player_x, player_y, Graphics.LEFT | Graphics.TOP);
                    // commander
                    player_x = (int)(Mission.ROOM_COORD[mission.players[mission.commander].inRoom << 1]*radar_zoom)-(selectorCommImageMini.getWidth() >> 1)-x-1;
                    player_y = (int)(Mission.ROOM_COORD[(mission.players[mission.commander].inRoom << 1)+1]*radar_zoom)-(selectorCommImageMini.getHeight() >> 1)-y-3;
                    gt.drawImage(selectorCommImageMini, player_x, player_y, Graphics.LEFT | Graphics.TOP);
                }
                radarImage = tmp;
            }
            ship = new Sprite(radarImage, RADAR_WIDTH, RADAR_HEIGHT);
            ship.setPosition(1, 1);
            ship.paint(g);
        }
        // draw radar scan effect
        if ((curr_frame % 80) < 31) {
            radarScan.setPosition(0, (curr_frame % 80)*8);
            radarScan.paint(g);
        }
        INIT_RADAR_pressAction.paint(g);
    }
    
    private final void MOVE_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 60) {
            wait = false;
            scrollStart = curr_frame;
        }
        
        if(radarActive && curr_frame == 25) {
            wait2 = false;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "PHASE 1:", 120, 144, g.TOP | g.HCENTER);
            fnt.drawString(g, "MOVE", 120, 144+fnt.getHeight()+3, g.TOP | g.HCENTER);
        } else {
            if (moveTo == -1) {
                drawScrollingString(g, "Press 1, 2 or 3 to move or OK to skip. ", 5, 48);
            } else {
                fntMini.drawString(g, "Press OK to validate.", 5, 48, g.TOP | g.LEFT);
            }
            // draw space ship
            int x = radar_x*(shipResizedImage.getWidth()-VIEW_WIDTH)/(shipImage.getWidth()-VIEW_WIDTH);
            int y = radar_y*(shipResizedImage.getHeight()-VIEW_HEIGHT)/(shipImage.getHeight()-VIEW_HEIGHT);
            radarImage = clip(shipResizedImage, x, y, VIEW_WIDTH, VIEW_HEIGHT);
            Image tmp = Image.createImage(radarImage.getWidth(), radarImage.getHeight());
            Graphics gt = tmp.getGraphics();
            gt.drawImage(radarImage, 0, 0, Graphics.LEFT | Graphics.TOP);
            if (!wait2 && radarActive) {
                paintProximityRadar(gt, x, y);
            }
            if ((curr_frame % 30) < 15) {
                if(moveTo == -1) {
                    for(int i=0; i<3; i++) {
                        int xb = Mission.ROOM_COORD[Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + i] << 1]-16-x;
                        int yb = Mission.ROOM_COORD[(Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + i] << 1)+1]-16-y;
                        gt.drawRegion(buttonsImage, 0, i << 5, 32, 32, Sprite.TRANS_NONE, xb, yb, Graphics.LEFT | Graphics.TOP);
                    }
                } else {
                    int xb2 = Mission.ROOM_COORD[Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + moveTo] << 1]-16-x;
                    int yb2 = Mission.ROOM_COORD[(Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + moveTo] << 1)+1]-16-y;
                    gt.drawRegion(buttonsImageRed, 0, moveTo << 5, 32, 32, Sprite.TRANS_NONE, xb2, yb2, Graphics.LEFT | Graphics.TOP);
                }
            }
            radarImage = tmp;
            // press OK flashing
            if ((curr_frame % 4) == 0)
                mainPressOK.nextFrame();
            mainPressOK.paint(g);
            if (radarCanActivate == 0) {
                if ((curr_frame % 4) == 0)
                    mainPressRadar.nextFrame();
                mainPressRadar.paint(g);
            }
            ship = new Sprite(radarImage, VIEW_WIDTH, VIEW_HEIGHT);
            ship.setPosition(1, 66);
            ship.paint(g);
            // radar scan effect
            if (radarActive && ((curr_frame % 75) < 21)) {
                radarScan.setPosition(0, (curr_frame % 75)*8 + 65);
                radarScan.paint(g);
            }
            
        }
    }
    
    private final void WAIT_PLAYERS012_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.setPosition(9, 8);
        faceIconSet.setFrame(mission.players[0].avatar);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, "WAITING FOR", 120, 134, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.getNumWait()+" PLAYER(S)", 120, 134+fnt.getHeight()+3, g.TOP | g.HCENTER);
        
        if ((curr_frame % 30) < 10) {
            fnt.drawString(g, "*", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
        } else if ((curr_frame % 30) < 20) {
            fnt.drawString(g, "**", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
        } else {
            fnt.drawString(g, "***", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
        }
    }
    
    private final void ACTION_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 60) {
            wait = false;
            scrollStart = curr_frame;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "PHASE 2:", 120, 144, g.TOP | g.HCENTER);
            fnt.drawString(g, "ACTION", 120, 144+fnt.getHeight()+3, g.TOP | g.HCENTER);
        } else {
            // help text
            if (actionToDo == -1)
                drawScrollingString(g, MissionType.getActionMsg(mission.players[0], mission.destroyed[mission.players[0].inRoom]), 5, 48);
            else
                drawScrollingString(g, "Press OK to validate or 1 to cancel action. ", 5, 48);
            // state of the room
            if (mission.destroyed[mission.players[0].inRoom])
                fntMini.drawString(g, "Room destroyed.", 4, 279, g.TOP | g.LEFT);
            else
                fntMini.drawString(g, "Room functional.", 4, 279, g.TOP | g.LEFT);
            // draw space ship
            int x = radar_x*(shipResizedImage.getWidth()-VIEW_WIDTH)/(shipImage.getWidth()-VIEW_WIDTH);
            int y = radar_y*(shipResizedImage.getHeight()-VIEW_HEIGHT)/(shipImage.getHeight()-VIEW_HEIGHT);
            radarImage = clip(shipResizedImage, x, y, VIEW_WIDTH, VIEW_HEIGHT);
            Image tmp = Image.createImage(radarImage.getWidth(), radarImage.getHeight());
            Graphics gt = tmp.getGraphics();
            gt.drawImage(radarImage, 0, 0, Graphics.LEFT | Graphics.TOP);
            if ((actionToDo != -1) && ((curr_frame % 30) < 15)) {
                int xb = Mission.ROOM_COORD[mission.players[0].inRoom << 1]-16-x;
                int yb = Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]-16-y;
                gt.drawRegion(actionIconsImage, 0, actionToDo << 5, 32, 32, Sprite.TRANS_NONE, xb, yb, Graphics.LEFT | Graphics.TOP);
            }
            radarImage = tmp;
            // press OK flashing
            if (moveTo != -1) {
                if ((curr_frame % 4) == 0)
                    mainPressOK.nextFrame();
                mainPressOK.paint(g);
            }
            ship = new Sprite(radarImage, VIEW_WIDTH, VIEW_HEIGHT);
            ship.setPosition(1, 66);
            ship.paint(g);
        }
    }
    
    private final void REACT_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        if (wait2)
            fnt.drawString(g, "--", 217, 26, g.TOP | g.HCENTER);
        else
            fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 60) {
            wait = false;
            scrollStart = curr_frame;
            wstart_frame = curr_frame-60;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "PHASE 3:", 120, 144, g.TOP | g.HCENTER);
            fnt.drawString(g, "REACTION", 120, 144+fnt.getHeight()+3, g.TOP | g.HCENTER);
        } else {
            if (wait2) {
                drawScrollingString(g, "Look and listen carefully to what is happening... ", 5, 48);
            } else {
                drawScrollingString(g, "Press OK to see stats. ", 5, 48);
                // press OK flashing
                if (moveTo != -1) {
                    if ((curr_frame % 4) == 0)
                        mainPressOK.nextFrame();
                    mainPressOK.paint(g);
                }
            }
            // draw space ship
            int x = radar_x*(shipResizedImage.getWidth()-VIEW_WIDTH)/(shipImage.getWidth()-VIEW_WIDTH);
            int y = radar_y*(shipResizedImage.getHeight()-VIEW_HEIGHT)/(shipImage.getHeight()-VIEW_HEIGHT);
            radarImage = clip(shipResizedImage, x, y, VIEW_WIDTH, VIEW_HEIGHT);
            Image tmp = Image.createImage(radarImage.getWidth(), radarImage.getHeight());
            Graphics gt = tmp.getGraphics();
            gt.drawImage(radarImage, 0, 0, Graphics.LEFT | Graphics.TOP);
            if (wstart_frame+60 == curr_frame) {
                int room_id = 0;
                if (react_id < 3) {
                    do {
                        react_id++;
                        if (react_id == -1)
                            room_id = mission.players[0].inRoom;
                        else if (react_id < 3)
                            room_id = Mission.CAN_MOVE_TO[mission.players[0].inRoom*3 + react_id];
                    } while (!mission.activity[room_id] && (react_id < 3));
                    if (react_id < 3) {
                        wstart_frame = curr_frame;
                        wave_x = Mission.ROOM_COORD[room_id << 1]-32;
                        wave_y = Mission.ROOM_COORD[(room_id << 1)+1]-32;
                        soundWave.setVisible(true);
                        int snd_id = mission.rand.nextInt(4)+2;
                        SoundManager.set3D(snd_id);
                        SoundManager.setPosition(snd_id, wave_x+32-Mission.ROOM_COORD[mission.players[0].inRoom << 1], wave_y+32+Mission.ROOM_COORD[(mission.players[0].inRoom << 1)+1]);
                        SoundManager.play(snd_id);
                        JunoMidlet.midlet.getDisplay().vibrate(1000);
                    } else {
                        soundWave.setVisible(false);
                        timer = PHASE_TIME >> 1;
                        wait2 = false;
                        scrollStart = curr_frame;
                    }
                } else {
                    soundWave.setVisible(false);
                    timer = PHASE_TIME >> 1;
                    wait2 = false;
                    scrollStart = curr_frame;
                }
            }
            soundWave.setPosition(wave_x-x, wave_y-y);
            soundWave.setFrame(((curr_frame-wstart_frame) >> 1) % 5);
            soundWave.paint(gt);
            radarImage = tmp;
            ship = new Sprite(radarImage, VIEW_WIDTH, VIEW_HEIGHT);
            ship.setPosition(1, 66);
            ship.paint(g);
        }
    }
    
    private final void STATS_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 40) {
            wait = false;
            scrollStart = curr_frame;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "GATHERING", 120, 134, g.TOP | g.HCENTER);
            fnt.drawString(g, "STATISTICS", 120, 134+fnt.getHeight()+3, g.TOP | g.HCENTER);
            
            if ((curr_frame % 15) < 5) {
                fnt.drawString(g, "*", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            } else if ((curr_frame % 15) < 10) {
                fnt.drawString(g, "**", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            } else {
                fnt.drawString(g, "***", 120, 134+fnt.getHeight()*2+8, g.TOP | g.HCENTER);
            }
        } else {
            // help text
            drawScrollingString(g, "Press OK to continue. ", 5, 48);
            // stats
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, "Statistics :", 8, 73, g.TOP | g.LEFT);
            fntMini.drawString(g, "Turn             : " + (mission.turn+1) + "/" + mission.type.num_turns, 8, 98, g.TOP | g.LEFT);
            fntMini.drawString(g, "People on ship   : " + mission.getAlive() + "/" + mission.type.num_players, 8, 98+fntMini.getHeight()+4, g.TOP | g.LEFT);
            fntMini.drawString(g, "Rooms functional : " + (mission.NUM_ROOMS-mission.getDestroyed()) + "/" + mission.NUM_ROOMS, 8, 98+(fntMini.getHeight() << 1)+8, g.TOP | g.LEFT);
            fntMini.drawString(g, "Rooms destroyed  : " + mission.getDestroyed() + "/" + mission.NUM_ROOMS, 8, 98+(fntMini.getHeight()*3)+12, g.TOP | g.LEFT);
            if ((mission.turn+1) % 3 == 0) {
                fntMini.drawString(g, "Technicians      : " + mission.getTechnicians() + "/" + mission.type.num_technicians, 8, 98+(fntMini.getHeight() << 2)+16, g.TOP | g.LEFT);
                fntMini.drawString(g, "Wreckers         : " + mission.getWreckers() + "/" + mission.type.num_wreckers, 8, 98+(fntMini.getHeight()*5)+20, g.TOP | g.LEFT);
                fntMini.drawString(g, "Aliens           : " + mission.getAliens() + "/" + mission.type.num_aliens, 8, 98+(fntMini.getHeight()*6)+24, g.TOP | g.LEFT);
            }
            
            // press OK flashing
            if ((curr_frame % 4) == 0)
                mainPressOK.nextFrame();
            mainPressOK.paint(g);
        }
    }
    
    private final void VOTE_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.setPosition(9, 8);
        faceIconSet.setFrame(mission.players[0].avatar);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 60) {
            wait = false;
            scrollStart = curr_frame;
            wstart_frame = curr_frame-60;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "PHASE 4:", 120, 144, g.TOP | g.HCENTER);
            fnt.drawString(g, "VOTE", 120, 144+fnt.getHeight()+3, g.TOP | g.HCENTER);
        } else {
            // help text
            drawScrollingString(g, "Choose a player to vote against, then press OK. ", 5, 48);
            // icons & names
            int x = 40, y = 72;
            for (int i=0; i < mission.type.num_players; i++) {
                if (i == 0) {
                    faceIconSet.setPosition(x, y);
                    faceIconSet.setFrame(8);
                    faceIconSet.paint(g);
                    fnt.drawString(g, "Blank vote", x+40, y+6, g.TOP | g.LEFT);
                    y += 34;
                } else if (mission.players[i].type != PlayerType.COMMANDER && mission.players[i].alive) {
                    faceIconSet.setPosition(x, y);
                    faceIconSet.setFrame(mission.players[i].avatar);
                    faceIconSet.paint(g);
                    fnt.drawString(g, mission.players[i].name, x+40, y+6, g.TOP | g.LEFT);
                    y += 34;
                }
            }
            // set selection arrow position
            arrow.setPosition(x - 36, 72 + 34*curr_vote);
            // selection arrow animation
            if ((curr_frame % 3) == 0)
                arrow.nextFrame();
            arrow.paint(g);
            // press OK flashing
            if ((curr_frame % 4) == 0)
                mainPressOK.nextFrame();
            mainPressOK.paint(g);
        }
    }
    
    private final void COMMANDER_ACTION_paint(Graphics g) {
        if (mission.players[0].type == PlayerType.COMMANDER) {
            main_bg.paint(g);
            faceIconSet.setPosition(9, 8);
            faceIconSet.setFrame(mission.players[0].avatar);
            faceIconSet.paint(g);
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
            fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
            
            // help text
            drawScrollingString(g, "Choose a player to eliminate, then press OK. ", 5, 48);
            // icons & names
            int x = 40, y = 72;
            for (int i=0; i < mission.type.num_players; i++) {
                if (i == 0) {
                    faceIconSet.setPosition(x, y);
                    faceIconSet.setFrame(8);
                    faceIconSet.paint(g);
                    fntMini.setStyle(Font.STYLE_BOLD);
                    fntMini.drawString(g, "Nobody", x+40, y+2, g.TOP | g.LEFT);
                    fntMini.setStyle(Font.STYLE_PLAIN);
                    int num = Math.min(mission.votes[i], curr_frame/4);
                    fntMini.drawString(g, "Votes: " + toBars(num) + " " + num, x+40, y+fntMini.getHeight()+2, g.TOP | g.LEFT);
                    y += 34;
                } else if (mission.players[i].type != PlayerType.COMMANDER && mission.players[i].alive) {
                    faceIconSet.setPosition(x, y);
                    faceIconSet.setFrame(mission.players[i].avatar);
                    faceIconSet.paint(g);
                    fntMini.setStyle(Font.STYLE_BOLD);
                    fntMini.drawString(g, mission.players[i].name, x+40, y+2, g.TOP | g.LEFT);
                    fntMini.setStyle(Font.STYLE_PLAIN);
                    int num = Math.min(mission.votes[i+1], curr_frame/4);
                    fntMini.drawString(g, "Votes: " + toBars(num) + " " + num, x+40, y+fntMini.getHeight()+2, g.TOP | g.LEFT);
                    y += 34;
                }
            }
            // set selection arrow position
            arrow.setPosition(x - 36, 72 + 34*curr_vote);
            // selection arrow animation
            if ((curr_frame % 3) == 0)
                arrow.nextFrame();
            arrow.paint(g);
            // press OK flashing
            if ((curr_frame % 4) == 0)
                mainPressOK.nextFrame();
            mainPressOK.paint(g);
        } else {
            main_bg.paint(g);
            faceIconSet.setPosition(9, 8);
            faceIconSet.setFrame(mission.players[0].avatar);
            faceIconSet.paint(g);
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
            fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
            
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "WAITING FOR", 120, 128, g.TOP | g.HCENTER);
            fnt.drawString(g, "THE COMMANDER", 120, 128+fnt.getHeight()+3, g.TOP | g.HCENTER);
            fnt.drawString(g, "TO TAKE A DECISION", 120, 128+(fnt.getHeight() << 1)+6, g.TOP | g.HCENTER);
            
            if ((curr_frame % 30) < 10) {
                fnt.drawString(g, "*", 120, 128+fnt.getHeight()*3+11, g.TOP | g.HCENTER);
            } else if ((curr_frame % 30) < 20) {
                fnt.drawString(g, "**", 120, 128+fnt.getHeight()*3+11, g.TOP | g.HCENTER);
            } else {
                fnt.drawString(g, "***", 120, 128+fnt.getHeight()*3+11, g.TOP | g.HCENTER);
            }
        }
    }
    
    private final void DECISION_paint(Graphics g) {
        main_bg.paint(g);
        faceIconSet.setPosition(9, 8);
        faceIconSet.setFrame(mission.players[0].avatar);
        faceIconSet.paint(g);
        fnt.setStyle(Font.STYLE_PLAIN);
        fnt.drawString(g, String.valueOf((int)timer)+"s", 217, 26, g.TOP | g.HCENTER);
        fnt.drawString(g, mission.players[0].name, 48, 7, g.TOP | g.LEFT);
        fnt.setStyle(Font.STYLE_BOLD);
        fnt.drawString(g, PlayerType.getRole(mission.players[0].type), 48, 7+fnt.getHeight()+2, g.TOP | g.LEFT);
        fntMini.setStyle(Font.STYLE_PLAIN);
        
        if(curr_frame == 60) {
            wait = false;
            scrollStart = curr_frame;
            wstart_frame = curr_frame-60;
        }
        
        if(wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            fnt.drawString(g, "PHASE 5:", 120, 144, g.TOP | g.HCENTER);
            fnt.drawString(g, "DECISION", 120, 144+fnt.getHeight()+3, g.TOP | g.HCENTER);
        } else {
            drawScrollingString(g, "Press OK to continue. ", 5, 48);
            // stats
            fnt.setStyle(Font.STYLE_BOLD);
            fnt.drawString(g, "Decision :", 8, 73, g.TOP | g.LEFT);
            // player eliminated
            if (mission.eliminate == -1) {
                faceIconSet.setPosition(8, 94);
                faceIconSet.setFrame(8);
                faceIconSet.paint(g);
                fntMini.setStyle(Font.STYLE_PLAIN);
                fntMini.drawString(g, "Nobody was chosen.", 48, 102, g.TOP | g.LEFT);
            } else {
                faceIconSet.setPosition(8, 94);
                faceIconSet.setFrame(mission.players[mission.eliminate].avatar);
                faceIconSet.paint(g);
                fntMini.setStyle(Font.STYLE_PLAIN);
                fntMini.drawString(g, "Eject " + mission.players[mission.eliminate].name, 48, 102, g.TOP | g.LEFT);
            }
            fnt.setStyle(Font.STYLE_PLAIN);
            if (mission.eliminate != 0) {
                if (mission.players[0].type == PlayerType.COMMANDER) {
                    fnt.drawString(g, "This is your", 120, 164, g.TOP | g.HCENTER);
                    fnt.drawString(g, "decision...", 120, 165+fnt.getHeight(), g.TOP | g.HCENTER);
                } else {
                    fnt.drawString(g, "You are safe...", 120, 168, g.TOP | g.HCENTER);
                }
            } else {
                fnt.drawString(g, "Goodbye...", 120, 168, g.TOP | g.HCENTER);
            }
            // press OK flashing
            if (moveTo != -1) {
                if ((curr_frame % 4) == 0)
                    mainPressOK.nextFrame();
                mainPressOK.paint(g);
            }
        }
    }
    
    private final void END_paint(Graphics g) {
        end_bg.paint(g);
        if (!mission.players[0].alive)
            death.paint(g);
        
        if(curr_frame == 40) {
            wait = false;
            SoundManager.play(1);
        }
        
        if(!wait) {
            fnt.setStyle(Font.STYLE_PLAIN);
            if (!mission.players[0].alive) {
                fnt.drawString(g, "You were ejected", 120, 158, g.TOP | g.HCENTER);
                fnt.drawString(g, "from the ship...", 120, 159+fnt.getHeight(), g.TOP | g.HCENTER);
            } else {
                if (mission.getWin() && mission.players[0].alive) {
                    fnt.drawString(g, "You win!", 120, 158, g.TOP | g.HCENTER);
                } else {
                    fnt.drawString(g, "You lose...", 120, 158, g.TOP | g.HCENTER);
                    fnt.drawString(g, MissionType.getWinMsg(MissionType.checkWin(mission.getAliens(), mission.getTechnicians(), mission.getWreckers(), mission.getDestroyed(), mission.turn)), 120, 160+fnt.getHeight(), g.TOP | g.HCENTER);
                }
            }
            // press OK flashing
            if (moveTo != -1) {
                if ((curr_frame % 4) == 0)
                    mainPressOK.nextFrame();
                mainPressOK.paint(g);
            }
        }
    }
    
}
