package game;

/**
 * MissionType.java
 * Mission types definitions.
 * @author Noda
 */
public final class MissionType {

    // win cases
    public static final int NOT_FINISHED = -1;
    public static final int GOODTEAM_WINS = 0;
    public static final int WRECKERS_WINS = 1;
    public static final int ALIENS_WINS = 2;
    
    // the differents missions
    public static final MissionType CLASSIC = new MissionType(
            6,
            10,
            "Classic",
            2,
            1,
            2,
            2
            );
    
    // action types
    public static final int ACTION_NOACTION = -1;
    public static final int ACTION_DESTROY = 0;
    public static final int ACTION_REPAIR = 1;
    
    // rules of a mission
    public int num_players;                 // number of players
    public int num_turns;                   // number of turns
    public int num_wreckers;                // number of wreckers
    public int num_aliens;                  // number of aliens
    public int num_technicians;             // number of technicians
    public String description;              // description of the mission
    public int radar_delay;                 // number of turns between radar activation is possible
    
    /** creates a new instance of MissionType */
    private MissionType(int num_players, int num_turns, String description, int num_wreckers, int num_aliens, int num_technicians, int radar_delay) {
        this.num_players = num_players;
        this.num_turns = num_turns;
        this.description = description;
        this.num_wreckers = num_wreckers;
        this.num_aliens = num_aliens;
        this.num_technicians = num_technicians;
        this.radar_delay = radar_delay;
    }
    
    /** returns the action message corresponding to the player specified */
    public final static String getActionMsg(Player p, boolean roomDestroyed) {
        switch(p.type) {
            case PlayerType.COMMANDER:
                return "Press OK to end phase. ";
            case PlayerType.ALIEN:
                if (roomDestroyed && p.actions_left[ACTION_REPAIR] != 0)
                    return "Press 1 to repair room (" + p.actions_left[ACTION_REPAIR] + " repair kit(s) left) or OK to end phase. ";
                else if (!roomDestroyed && p.actions_left[ACTION_DESTROY] != 0)
                    return "Press 1 to destroy room (" + p.actions_left[ACTION_DESTROY] + " bomb kit(s) left) or OK to end phase. ";
                else
                    return "Press OK to end phase. ";
            case PlayerType.TECHNICIAN:
                if (roomDestroyed && p.actions_left[ACTION_REPAIR] != 0)
                    return "Press 1 to repair room (" + p.actions_left[ACTION_REPAIR] + " repair kit(s) left) or OK to end phase. ";
                else
                    return "Press OK to end phase. ";
            case PlayerType.WRECKER:
                if (!roomDestroyed && p.actions_left[ACTION_DESTROY] != 0)
                    if (p.actions_left[ACTION_DESTROY] < 0)
                        return "Press 1 to destroy room or OK to end phase. ";
                    else
                        return "Press 1 to destroy room (" + p.actions_left[ACTION_DESTROY] + " bomb kit(s) left) or OK to end phase. ";
                else
                    return "Press OK to end phase. ";
        }
        return "";
    }
    
    /** returns the action code corresponding to the player specified and the key pressed */
    public final static int getActionCode(Player p, boolean roomDestroyed, int key) {
        if (key == 1) {
            switch(p.type) {
                case PlayerType.ALIEN:
                    if (roomDestroyed && p.actions_left[ACTION_REPAIR] != 0)
                        return ACTION_REPAIR;
                    else if (!roomDestroyed && p.actions_left[ACTION_DESTROY] != 0)
                        return ACTION_DESTROY;
                    else
                        return ACTION_NOACTION;
                case PlayerType.TECHNICIAN:
                    if (roomDestroyed && p.actions_left[ACTION_REPAIR] != 0)
                        return ACTION_REPAIR;
                    else
                        return ACTION_NOACTION;
                case PlayerType.WRECKER:
                    if (!roomDestroyed && p.actions_left[ACTION_DESTROY] != 0)
                        return ACTION_DESTROY;
                    else
                        return ACTION_NOACTION;
            }
        }
        return ACTION_NOACTION;
    }
    
    /** check if a team win the game */
    public final static int checkWin(int nb_aliens, int nb_tech, int nb_wreckers, int destroyed, int turn) {
        if (turn == CLASSIC.num_turns)
            return GOODTEAM_WINS;
        else if (destroyed > Mission.NUM_ROOMS/2)
            return WRECKERS_WINS;
        else if (nb_aliens == 0 && nb_wreckers == 0)
            return GOODTEAM_WINS;
        else if (nb_tech == 0 && nb_wreckers == 0)
            return ALIENS_WINS;
        return NOT_FINISHED;
    }
    
    /** returns win message for the given winning team */
    public final static String getWinMsg(int team) {
        switch(team) {
            case MissionType.GOODTEAM_WINS:
                return "Juno team wins!";
            case MissionType.ALIENS_WINS:
                return "The alien wins!";
            case MissionType.WRECKERS_WINS:
                return "Wreckers wins!";
        }
        return "";
    }
}
