package game;

import java.io.DataInputStream;
import java.io.IOException;


/**
 * Reader.java
 * Class to read every received data.
 * @author Drag-on
 */
public class Reader implements Runnable {
    
    // attributes
    public DataInputStream in;              // read buffer
    public boolean isConnected=true;        // true if connection is alive
    public Thread r;                        // current thread
    private int ordre=-1;                   // order to be executed by the thread
    private Mission mission=null;           // object mission to modify
    
    /** Creates a new instance of Reader */
    public Reader(){
        r=new Thread(this);
        r.start();
    }
    
    /** main loop for the thread */
    public void run() {
        while (isConnected){
            synchronized(r){
                try {
                    r.wait();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
            switch (ordre){
                case 0:wait_players0(mission);break;
                case 1:wait_players1(mission);break;
                case 2:wait_players2(mission);break;
                case 3:wait_commander(mission);break;
                default:break;}
        }
    }
    
    /** method called by mission to give orders */
    public void wake(int ordre, Mission m) {
        synchronized(r){
            this.ordre=ordre;
            mission=m;
            r.notify();
        }
    }
    
    /** method to read from the buffer a byte array of size i */
    public byte[] readBytes(int i){
        String s=null;
        byte[] b = new byte[i];
        try {
            in.read(b);
        } catch (IOException e) {
            System.out.println("dconnection...");
        }
        if (isConnected){
            return b;
        }
        return null;
    }
    
    /** first synchronisation phase between server and clients */
    private void wait_players0(Mission m){
        if (m.isServer){
            byte[] b=readBytes(3);
            int id=(int)b[1];
            m.players[id].inRoom=(int)b[2];
            boolean last=m.setPlayersReady(false);
            if (last) {
                m.writeRoom();m.setPlayersReady(true);}
        } else {
            byte[] b0=new byte[3];
            b0[1]=(byte)m.players[0].server_id;
            b0[2]=(byte)m.players[0].inRoom;
            m.writer.write(b0);
            try {
                while (in.available()<7){
                    synchronized(this){
                        this.wait(100);}
                    System.out.println(in.available());
                }
            } catch (Exception ex) {
                ex.printStackTrace();}
            byte[] b=readBytes(7);
            int id;
            int room;
            int i=1;
            while (i<7){
                room=(int)b[i];
                System.out.println("Room "+room);
                for (int p=0;p<6;p++) {
                    if (m.players[p].server_id==(i-1)) m.players[p].inRoom=room;}
                i++;
            }
            m.setPlayersReady(true);
        }
    }
    
    /** second synchronisation phase between server and clients */
    private void wait_players1(Mission m){
        if (m.isServer){
            byte[] b=readBytes(4);
            int room=(int)b[1];
            int activity=(int)b[2];
            int destr=(int)b[3];
            if (activity==1)m.activity[room]=true;
            else m.activity[room]=false;
            if (destr==1)m.destroyed[room]=true;
            else m.destroyed[room]=false;
            boolean last=m.setPlayersReady(false);
            if (last) {
                m.writeActivity();m.setPlayersReady(true);}
        } else {
            byte[] b0=new byte[4];
            b0[1]=(byte)m.players[0].inRoom;
            if (m.activity[m.players[0].inRoom])
                b0[2]=(byte)1;
            else
                b0[2]=(byte)0;
            if (m.destroyed[m.players[0].inRoom])
                b0[3]=(byte)1;
            else
                b0[3]=(byte)0;
            m.writer.write(b0);
            try {
                while (in.available()<21){
                    synchronized(this){
                        this.wait(100);}
                    System.out.println(in.available());
                }
            } catch (Exception ex) {
                ex.printStackTrace();}
            byte[] b=readBytes(21);
            int activity;
            int destroyed;
            int i=1;
            int j=0;
            while (i<21){
                activity=(int)b[i];
                System.out.println("Activity "+activity);
                i++;
                destroyed=(int)b[i];
                System.out.println("Destroyed "+destroyed);
                i++;
                if (activity==1)m.activity[j]=true;
                else m.activity[j]=false;
                if (destroyed==1)m.destroyed[j]=true;
                else m.destroyed[j]=false;
                j++;
            }
            m.setPlayersReady(true);
        }
    }
    
    /** third synchronisation phase between server and clients */
    private void wait_players2(Mission m){
        if (m.isServer){
            byte[] b=readBytes(3);
            int id=(int)b[1];
            int vote=(int)b[2];
            if (vote==-1){
                m.players[id].vote=-1;
                System.out.println(m.players[id].name+" vote blanc");} else {m.players[id].vote=vote;
                System.out.println(m.players[id].name+" vote contre "+m.players[vote].name);}
            boolean last=m.setPlayersReady(false);
            if (last) {
                m.writeVote();m.setPlayersReady(true);}
        } else {
            byte[] b0=new byte[3];
            b0[1]=(byte)m.players[0].server_id;
            int vote=m.players[0].vote;
            if (vote==-1){
                System.out.println("vote blanc" + vote);
                b0[2]=(byte)-1;} else{
                System.out.println(m.players[0].name+ " vote contre "+m.players[vote].name);
                b0[2]=(byte)m.players[vote].server_id;}
            m.writer.write(b0);
            try {
                while (in.available()<1){
                    synchronized(this){
                        this.wait(100);}
                    System.out.println(in.available());
                }
            } catch (Exception ex) {
                ex.printStackTrace();}
            byte[] b=readBytes(1);
            m.setPlayersReady(true);
        }
    }
    
    /** fourth synchronisation phase between server and clients */
    private void wait_commander(Mission m){
        byte[] b=readBytes(2);
        int elim=b[1];
        for (int i=0;i<m.type.num_players;i++){
            if (m.players[i].server_id==elim) m.eliminate=i;}
        m.setPlayersReady(true);
    }
    
    /** disconnect */
    public void disconnect(){
        try {
            isConnected=false;
            in.close();
            wake(-1,null);
        } catch (IOException ex) {
            System.out.println("Erreur de fermeture du reader");
        }
    }
}
