package game;

import java.io.InputStream;
import java.util.Vector;
import javax.microedition.amms.EffectModule;
import javax.microedition.amms.GlobalManager;
import javax.microedition.amms.SoundSource3D;
import javax.microedition.amms.Spectator;
import javax.microedition.amms.control.audio3d.DistanceAttenuationControl;
import javax.microedition.amms.control.audio3d.LocationControl;
import javax.microedition.media.Manager;
import javax.microedition.media.Player;
import javax.microedition.media.control.VolumeControl;

/**
 * SoundManager.java
 * Sound manager of the game
 * @author Flocoon
 */
public class SoundManager {
    
    static public boolean sound_enabled = true;
    static private Player players[];
    static private SoundSource3D sound3D[];
    static private Spectator spectator;
    static private VolumeControl volume[];
    static private LocationControl location[];
    
    /** initialize the sound manager */
    public final static void initSoundManager() {
        try{
            players = new Player[8];
            volume = new VolumeControl[8];
            sound3D = new SoundSource3D[8];
            location = new LocationControl[8];
            spectator = GlobalManager.getSpectator();
            LocationControl speC = (LocationControl)spectator.getControl("javax.microedition.amms.control.audio3d.LocationControl");
            speC.setCartesian(0, 0, 0);
        } catch(Exception e){
            System.out.println("Error during soundinit");
        }
    }
    
    /** create or replace one of the players, simple version */
    public final static void createPlayer(int numero, String fileName) {
        createPlayer(numero, fileName, true);
    }
    
    /** create or replace one of the players */
    public final static void createPlayer(int numero, String fileName, boolean prepare) {
        try {
            if(players[numero] != null){
                freePlayer(numero);
            }
            InputStream is = SoundManager.class.getResourceAsStream(fileName);
            Player p = null;
            if (fileName.endsWith("mp3")) {
                p = Manager.createPlayer(is,"audio/mpeg");
            } else if (fileName.endsWith("wav")) {
                p = Manager.createPlayer(is,"audio/x-wav");
            } else {
                p = Manager.createPlayer(is,"audio/amr");
            }
            players[numero] = p;
            if (prepare) {
                p.realize();
                p.prefetch();
                volume[numero] = (VolumeControl)p.getControl("VolumeControl");
            }
        } catch(Exception e) {
            System.out.println("Error while creating the player n"+numero);
        }
    }
    
    public final static void set3D(int numPlayer){
        try {
            if (sound3D[numPlayer] == null) {
                sound3D[numPlayer] = GlobalManager.createSoundSource3D();
                location[numPlayer] = (LocationControl)sound3D[numPlayer].getControl("javax.microedition.amms.control.audio3d.LocationControl");
                DistanceAttenuationControl distC = (DistanceAttenuationControl)sound3D[numPlayer].getControl("javax.microedition.amms.control.audio3d.DistanceAttenuationControl");
                distC.setParameters(100, 5000, false, 1000);
                sound3D[numPlayer].addPlayer(players[numPlayer]);
                players[numPlayer].realize();
                players[numPlayer].prefetch();
                volume[numPlayer] = (VolumeControl)players[numPlayer].getControl("VolumeControl");
            }
        } catch(Exception e) {
            System.out.println(e);
        }
    }
    
    public final static void freePlayer(int numPlayer){
        try {
            if (players[numPlayer] != null) {
                sound3D[numPlayer] = null;
                location[numPlayer] = null;
                stop(numPlayer);
                players[numPlayer].deallocate();
                players[numPlayer] = null;
            }
            System.gc();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    
    /** set the 3D position of a player */
    public final static void setPosition(int numPlayer, int x, int z) {
        location[numPlayer].setCartesian(x, 0, z);
    }
    
    /** start a player */
    public final static void play(int numPlayer) {
        try {
            if (sound_enabled)
                players[numPlayer].start();
        } catch(Exception e) {
            System.out.println("Error while playing sound");
        }
    }
    
    /** return true is player is active (playing) */
    public final static boolean isPlaying(int numPlayer) {
        try {
            return (players[numPlayer].getState() == Player.STARTED);
        } catch (Exception e) {}
        return false;
    }
    
    /** stop and rewind a player */
    public final static void stop(int numPlayer) {
        try {
            if (players[numPlayer].getState() == Player.STARTED) {
                try {
                    players[numPlayer].stop();
                    players[numPlayer].setMediaTime(0);
                } catch (Exception e) {
                    System.out.println(e);
                }
            }
        } catch(Exception e) {}
    }
    
    /** set the volume of the player (numplayer) to (value) */
    public final static void setVolume(int numPlayer, int value){
        try {
            volume[numPlayer].setLevel(value);
        } catch (Exception e) {}
    }
    
    /** set loop status of a player */
    public final static void setLoop(int numPlayer, boolean loop){
        try {
            if (loop)
                players[numPlayer].setLoopCount(-1);
            else
                players[numPlayer].setLoopCount(0);
        } catch (Exception e) {}
    }
    
    /** prevents instanciation */
    private SoundManager() {}
    
}
